/*
 * Copyright (c) 2024 Oray Inc. All rights reserved.
 *
 * No Part of this file may be reproduced, stored
 * na retrieval system, or transmitted, in any form, or by any means,
 * electronic, mechanical, photocopying, recording, or otherwise,
 * without the prior consent of Oray Inc.
 *
 * @author wuwenze
 */

package sunapi

import (
	"crypto/hmac"
	"crypto/md5"
	"crypto/sha256"
	"encoding/base64"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"github.com/google/uuid"
	"github.com/pkg/errors"
	"io"
	"net/http"
	"strconv"
	"strings"
	"sunlogin-api-example-golang/sunapi/dto"
	"sunlogin-api-example-golang/sunapi/entity"
	"time"
)

// SunloginAuthClient 提供API集成，获取授权码的能力
type SunloginAuthClient struct {
	Host      string // 授权服务地址
	ApiKey    string // ApiKey，参见：【系统管理】-【API集成许可】
	ApiSecret string // ApiSecret, 参见：【系统管理】-【API集成许可】
}

// GetAuthorizationCode 获取授权码
// https://sunlogin.apifox.cn/api-150317885
func (sac *SunloginAuthClient) GetAuthorizationCode(req *dto.GetAuthorizationCodeRequest) (result *entity.AuthCode, err error) {
	authUrl := fmt.Sprintf("%s/keyguard/authorization_code", sac.Host)

	marshal, err := json.Marshal(req)
	if err != nil {
		return nil, errors.Wrap(err, "marshal body fail")
	}
	httpRequest, err := sac.createSignatureRequest(http.MethodPost, authUrl, marshal)
	if err != nil {
		return nil, errors.Wrapf(err, "create signature request fail, authUrl=%s", authUrl)
	}
	httpResponse, err := NewHttpClient().Do(httpRequest)
	if err != nil {
		return nil, errors.Wrap(err, "send request fail")
	}

	defer httpResponse.Body.Close()
	responseBody, err := io.ReadAll(httpResponse.Body)
	if err != nil {
		return nil, errors.Wrap(err, "read response body fail")
	}
	if httpResponse.StatusCode != http.StatusOK {
		return nil, errors.Errorf("%s", responseBody)
	}
	if err = json.Unmarshal(responseBody, &result); err != nil {
		return nil, errors.Wrap(err, "unmarshal response body fail")
	}
	return result, nil
}

// createSignatureRequest 创建签名请求
func (sac *SunloginAuthClient) createSignatureRequest(method string, url string, body []byte) (*http.Request, error) {
	// 准备签名所需参数
	bodyHash := md5.Sum(body)
	contentMd5 := hex.EncodeToString(bodyHash[:])
	timestamp := strconv.FormatInt(time.Now().Unix(), 10)
	nonce := uuid.New().String()

	// 计算签名
	hash := hmac.New(sha256.New, []byte(sac.ApiSecret))
	hash.Write([]byte(fmt.Sprintf("%s\n%s\n%s\n", contentMd5, timestamp, nonce)))
	signature := base64.StdEncoding.EncodeToString(hash.Sum(nil))

	// 构建请求
	httpRequest, _ := http.NewRequest(method, url, strings.NewReader(string(body)))
	// 传入相关参数到Header
	httpRequest.Header.Add("Content-Md5", contentMd5)
	httpRequest.Header.Add("X-Ca-Api-Key", sac.ApiKey)
	httpRequest.Header.Add("X-Ca-Timestamp", timestamp)
	httpRequest.Header.Add("X-Ca-Nonce", nonce)
	httpRequest.Header.Add("X-Ca-Signature", signature)
	return httpRequest, nil
}