/*
 * Copyright (c) 2024 Oray Inc. All rights reserved.
 *
 * No Part of this file may be reproduced, stored
 * na retrieval system, or transmitted, in any form, or by any means,
 * electronic, mechanical, photocopying, recording, or otherwise,
 * without the prior consent of Oray Inc.
 *
 * @author wuwenze
 */

package sunapi

import (
	"bytes"
	"crypto/aes"
	"encoding/base64"
)

// SunloginApiCipher 负责对API的报文进行加解密操作
// https://sunlogin.apifox.cn/doc-3780603
type SunloginApiCipher struct {
	AesKey []byte
}

// Encrypt 执行加密操作
func (sae *SunloginApiCipher) Encrypt(src []byte) (string, error) {
	block, err := aes.NewCipher(sae.AesKey)
	if err != nil {
		return "", err
	}
	blockSize := block.BlockSize()
	src = pkcs7Padding(src, blockSize)
	encrypted := make([]byte, len(src))
	for bs, be := 0, blockSize; bs < len(src); bs, be = bs+blockSize, be+blockSize {
		block.Encrypt(encrypted[bs:be], src[bs:be])
	}
	return base64.StdEncoding.EncodeToString(encrypted), nil
}

// Decrypt 执行解密操作
func (sae *SunloginApiCipher) Decrypt(encryptedStr string) ([]byte, error) {
	encrypted, err := base64.StdEncoding.DecodeString(encryptedStr)
	if err != nil {
		return []byte{}, err
	}

	block, err := aes.NewCipher(sae.AesKey)
	if err != nil {
		return []byte{}, err
	}
	blockSize := block.BlockSize()
	decrypted := make([]byte, len(encrypted))
	for bs, be := 0, blockSize; bs < len(encrypted); bs, be = bs+blockSize, be+blockSize {
		block.Decrypt(decrypted[bs:be], encrypted[bs:be])
	}
	return pkcs7UnPadding(decrypted), nil
}

// pkcs7Padding 实现PKCS#7填充
func pkcs7Padding(ciphertext []byte, blockSize int) []byte {
	padding := blockSize - len(ciphertext)%blockSize
	padtext := bytes.Repeat([]byte{byte(padding)}, padding)
	return append(ciphertext, padtext...)
}

// pkcs7UnPadding 实现PKCS#7填充的移除（解密时使用）
func pkcs7UnPadding(origData []byte) []byte {
	length := len(origData)
	unpadding := int(origData[length-1])
	return origData[:(length - unpadding)]
}