/*
 * Copyright (c) 2024 Oray Inc. All rights reserved.
 *
 * No Part of this file may be reproduced, stored
 * na retrieval system, or transmitted, in any form, or by any means,
 * electronic, mechanical, photocopying, recording, or otherwise,
 * without the prior consent of Oray Inc.
 *
 * @author wuwenze
 */

package sunapi

import (
	"encoding/json"
	"fmt"
	"github.com/pkg/errors"
	"io"
	"net/http"
	"net/url"
	"os/exec"
	"reflect"
	"runtime"
	"strconv"
	"strings"
	"sunlogin-api-example-golang/sunapi/dto"
	"sunlogin-api-example-golang/sunapi/entity"
)

// SunloginControlClient 向日葵主控端API
type SunloginControlClient struct {
	cli *sunloginClientApi
}

func NewSunloginControlClient(host string, authCli *SunloginAuthClient) *SunloginControlClient {
	return &SunloginControlClient{cli: &sunloginClientApi{
		host:    host,
		authCli: authCli,
	}}
}

// LaunchApp 调起主控端程序
func (scrc *SunloginControlClient) LaunchApp() error {
	if runtime.GOOS != "windows" {
		return fmt.Errorf("launchApp only works on Windows, self: %s", runtime.GOOS)
	}
	cmd := exec.Command("cmd", "/c", "start", "sl-remote://")
	if err := cmd.Run(); err != nil {
		return errors.Wrapf(err, "launch app fail")
	}
	return nil
}

// IsAlive 检测主控端程序是否存活
// https://sunlogin.apifox.cn/api-151875839
func (scrc *SunloginControlClient) IsAlive() error {
	result, err := scrc.cli.sendRequest(http.MethodGet, "/is_alive", url.Values{}, map[string]string{})
	if err != nil {
		return errors.Wrapf(err, "send request fail")
	}

	if result.HttpStatus != 200 {
		return fmt.Errorf("check is alive fail, [status=%d, response=%s]", result.HttpStatus, result.HttpResponse)
	}
	return nil
}

// LocateHost 在设备列表中定位指定设备ID并发起远控
// https://sunlogin.apifox.cn/api-152488517
func (scrc *SunloginControlClient) LocateHost(req *dto.LocateHostRequest) error {
	formData, _ := structToMap(req)
	result, err := scrc.cli.sendRequest(http.MethodPost, "/locate_host", url.Values{}, formData)
	if err != nil {
		return errors.Wrapf(err, "send request fail")
	}
	if result.Code != 200 {
		return fmt.Errorf("locate host fail, [status=%d, code=%d, message=%s]", result.HttpStatus, result.Code, result.Message)
	}
	return nil
}

// RemoteControl 指定设备ID/识别码发起远控
// https://sunlogin.apifox.cn/api-150555663
func (scrc *SunloginControlClient) RemoteControl(req *dto.RemoteControlRequest) error {
	formData, _ := structToMap(req)
	result, err := scrc.cli.sendRequest(http.MethodPost, "/remote_control", url.Values{}, formData)
	if err != nil {
		return errors.Wrapf(err, "send request fail")
	}
	// 发起远控失败
	if result.Code != 200 {
		return fmt.Errorf("remote control fail, [status=%d, code=%d, message=%s]", result.HttpStatus, result.Code, result.Message)
	}
	return nil
}

// SunloginRemoteClient 向日葵被控端API
type SunloginRemoteClient struct {
	cli *sunloginClientApi
}

func NewSunloginRemoteClient(host string, authCli *SunloginAuthClient) *SunloginRemoteClient {
	return &SunloginRemoteClient{cli: &sunloginClientApi{
		host:    host,
		authCli: authCli,
	}}
}

// GetDeviceInfo 获取当前设备信息
// https://sunlogin.apifox.cn/api-150555740
func (src *SunloginRemoteClient) GetDeviceInfo() (*entity.DeviceInfo, error) {
	result, err := src.cli.sendRequest(http.MethodGet, "/get_deviceinfo", url.Values{}, map[string]string{})
	if err != nil {
		return nil, errors.Wrapf(err, "send request fail")
	}

	// 获取设备信息失败
	if result.Code != 200 {
		return nil, fmt.Errorf("get remote device info fail, [status=%d, code=%d, message=%s]", result.HttpStatus, result.Code, result.Message)
	}

	// 解析设备信息
	var deviceInfo entity.DeviceInfo
	if err = json.Unmarshal(result.DecryptedData, &deviceInfo); err != nil {
		return nil, errors.Wrap(err, "unmarshal response data fail")
	}
	return &deviceInfo, nil
}

// sunloginClientApiResult 向日葵客户端API响应结构
type sunloginClientApiResult struct {
	Code          int32  `json:"code"`    // 返回状态码
	Message       string `json:"message"` // 消息
	Data          string `json:"data"`    // 原始Data字符串
	DecryptedData []byte `json:"-"`       // 解密后的Data
	HttpStatus    int    `json:"-"`       // HTTP请求状态码
	HttpResponse  []byte `json:"-"`       // HTTP请求响应原文
}

// sunloginClientApi 向日葵客户端Api
type sunloginClientApi struct {
	host    string              // 桌面端Api地址
	authCli *SunloginAuthClient // 授权服务器API
}

// sendRequest 发送业务请求
func (sca *sunloginClientApi) sendRequest(method, path string, query url.Values, formData map[string]string) (*sunloginClientApiResult, error) {
	// 创建请求
	httpRequest, cipher, err := sca.createAuthorizedRequest(method, path, query, formData)
	if err != nil {
		return nil, errors.Wrap(err, "create request fail")
	}

	// 发送请求
	httpResponse, err := NewHttpClient().Do(httpRequest)
	if err != nil {
		return nil, errors.Wrap(err, "send request fail")
	}
	defer httpResponse.Body.Close()

	// 解析响应
	responseBody, err := io.ReadAll(httpResponse.Body)
	if err != nil {
		return nil, errors.Wrap(err, "read response body fail")
	}
	res := &sunloginClientApiResult{
		HttpStatus:   httpResponse.StatusCode,
		HttpResponse: responseBody,
	}
	if err = json.Unmarshal(responseBody, &res); err != nil {
		return nil, errors.Wrap(err, "unmarshal response body fail")
	}
	// Data字段需要解密处理
	if res.Code == 200 && res.Data != "" {
		decryptedData, err := cipher.Decrypt(res.Data)
		if err != nil {
			return res, errors.Wrap(err, "decrypt response data fail")
		}
		res.DecryptedData = decryptedData
	}
	return res, nil
}

// createAuthorizedRequest 创建已授权的请求
func (sca *sunloginClientApi) createAuthorizedRequest(method, path string, query url.Values, formData map[string]string) (*http.Request, *SunloginApiCipher, error) {
	// 获取授权码
	authCode, err := sca.authCli.GetAuthorizationCode(&dto.GetAuthorizationCodeRequest{Method: method, Path: path})
	if err != nil {
		return nil, nil, errors.Wrap(err, "get auth code fail")
	}
	// 构建业务请求
	cipher := &SunloginApiCipher{AesKey: []byte(authCode.AesKey)}
	targetUrl, err := url.Parse(fmt.Sprintf("%s%s", sca.host, path))
	if err != nil {
		return nil, nil, errors.Wrap(err, "build targetUrl fail")
	}
	targetUrl.RawQuery = query.Encode()

	// 参数值加密(x-www-form-urlencoded)
	payload := ""
	for k, v := range formData {
		if v == "" {
			continue
		}
		encryptValue, err := cipher.Encrypt([]byte(v))
		if err != nil {
			return nil, nil, errors.Wrapf(err, "encrypt formdata[%s] fail", k)
		}
		payload += fmt.Sprintf("%s=%s&", k, encryptValue)
	}

	// 构建请求
	httpRequest, err := http.NewRequest(method, targetUrl.String(), strings.NewReader(payload))
	if err != nil {
		return nil, nil, errors.Wrap(err, "build http request fail")
	}
	httpRequest.Header.Add("Authorization", fmt.Sprintf("Bearer %s", authCode.Code))
	return httpRequest, cipher, nil
}

// structToMap 将结构体转换为map[string]string，使用json标签作为key
func structToMap(obj interface{}) (map[string]string, error) {
	result := make(map[string]string)

	t := reflect.TypeOf(obj)
	v := reflect.ValueOf(obj)
	if t.Kind() == reflect.Ptr {
		t = t.Elem()
		v = v.Elem()
	}
	if t.Kind() != reflect.Struct {
		return nil, fmt.Errorf("expected struct, got %s", t.Kind())
	}

	for i := 0; i < t.NumField(); i++ {
		field := t.Field(i)
		fieldValue := v.Field(i)
		tag := field.Tag.Get("json")
		if tag == "" || tag == "-" {
			tag = field.Name
		}

		// 处理json标签中的逗号（如`json:"fieldname,omitempty"`）
		if commaIdx := strings.Index(tag, ","); commaIdx != -1 {
			tag = tag[:commaIdx]
		}

		// 将字段值转换为字符串
		var valueStr string
		switch fieldValue.Kind() {
		case reflect.String:
			valueStr = fieldValue.String()
		case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
			valueStr = strconv.FormatInt(fieldValue.Int(), 10)
		case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64:
			valueStr = strconv.FormatUint(fieldValue.Uint(), 10)
		case reflect.Float32, reflect.Float64:
			valueStr = strconv.FormatFloat(fieldValue.Float(), 'f', -1, 64)
		case reflect.Bool:
			valueStr = strconv.FormatBool(fieldValue.Bool())
		default:
			continue
		}
		result[tag] = valueStr
	}

	return result, nil
}